package com.example.authapi.security.jwt;

import com.example.authapi.users.application.dto.UserDTO;
import com.example.authapi.users.domain.entity.UserDomain;
import io.jsonwebtoken.*;
import io.jsonwebtoken.io.Decoders;
import io.jsonwebtoken.security.Keys;
import io.jsonwebtoken.security.SignatureException;
import jakarta.annotation.PostConstruct;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.stereotype.Component;
import java.security.Key;
import java.util.*;
import java.util.stream.Collectors;

@Component
@Slf4j
public class JwtProvider {
    @Value("${jwt.secret}")
    private String secret;

    @PostConstruct
    protected void init() {
        log.info("init...");
        secret = Base64.getEncoder().encodeToString(secret.getBytes());
        log.info("...init");
    }

    public String createToken(UserDTO userDTO) {
        log.info("createToken...");
        Map<String, Object> claims;
        claims = Jwts.claims().setSubject(userDTO.getUsername());
        claims.put("id", userDTO.getId());
        claims.put("roles", userDTO.getRoles().stream().map(rol -> rol.getName()).collect(Collectors.toList()));
        claims.put("email", userDTO.getEmail());
        Date now = new Date();
        Date exp = new Date(now.getTime() + 3600000);
        log.info("...createToken");
        return Jwts.builder()
                .setClaims(claims)
                .setIssuedAt(now)
                .setExpiration(exp)
                .signWith(SignatureAlgorithm.HS256, secret)
                .compact();
    }

    public boolean validateToken(String token) {
        log.info("validateToken...");
        try {
            // Jwts.parser().setSigningKey(secret).parseClaimsJws(token)
            Jwts.parserBuilder().setSigningKey(secret).build().parseClaimsJws(token).getBody();
            log.info("...validateToken1");
            return true;
        } catch (ExpiredJwtException e) {
            log.error("expired token");
        } catch (UnsupportedJwtException e) {
            log.error("unsupported token");
        } catch (MalformedJwtException e) {
            log.error("malformed token");
        } catch (SignatureException e) {
            log.error("bad signature");
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (Exception e) {
            log.error("fail token");
        }
        log.info("...validateToken2");
        return false;
    }

    public String getUserNameFromToken(String token) {
        log.info("getUserNameFromToken...");
        try {
            log.info("...getUserNameFromToken1");
            return Jwts.parserBuilder().setSigningKey(secret).build().parseClaimsJws(token).getBody().getSubject();
        }catch (Exception e) {
            log.info("...getUserNameFromToken2");
            return "token erroneo";
        }
    }

    private List<String> getRoles(UserDomain principal) {
        log.info("...getRoles...");
        return principal.getAuthorities()
                .stream().map(GrantedAuthority::getAuthority).collect(Collectors.toList());
    }

    private Key getKey(String secret){
        log.info("getKey...");
        byte [] secretBytes = Decoders.BASE64URL.decode(secret);
        log.info("...getKey");
        return Keys.hmacShaKeyFor(secretBytes);
    }
}
